@@ -1,5 +1,5 @@
 package PAR;
-$PAR::VERSION = '0.949_01';
+$PAR::VERSION = '0.969_01';
 
 use 5.006;
 use strict;
@@ -13,18 +13,16 @@ PAR - Perl Archive Toolkit
 
 =head1 VERSION
 
-This document describes version 0.949_01 of PAR, released August 5, 2006.
-
-This is a developer release. It contains many new changes and might
-introduce new bugs. It is intended for testing purposes.
+This document describes version 0.97 of PAR, released November 29, 2006.
 
 =head1 SYNOPSIS
 
 (If you want to make an executable that contains all module, scripts and
-data files, please consult the bundled L<pp> utility instead.)
+data files, please consult the L<pp> utility instead. L<pp> used to be
+part of the PAR distribution but is not shipped as part of the L<PAR::Packer>
+distribution instead.)
 
-Following examples assume a F<foo.par> file in Zip format; support for
-compressed tar (F<*.tgz>/F<*.tbz2>) format is under consideration.
+Following examples assume a F<foo.par> file in Zip format.
 
 To use F<Hello.pm> from F<./foo.par>:
 
@@ -109,19 +107,31 @@ If you have L<PAR::Repository::Client> installed, you can do this:
 
 And PAR will fetch any modules you don't have from the specified PAR
 repository. For details on how this works, have a look at the SEE ALSO
-section below.
+section below. Instead of an URL or local path, you can construct an
+L<PAR::Repository::Client> object manually and pass that to PAR.
+If you specify the C<install =E<gt> 1> option in the C<use PAR>
+line above, the distribution containing C<Module> will be permanently
+installed on your system. (C<use PAR { repository =E<gt> 'http://foo/bar', install =E<gt> 1 };>)
+
+Finally, you can combine the C<run> and C<repository>
+options to run an application directly from a repository! (And you can add
+the C<install> option, too.)
+
+  use PAR { repository => 'http://foo/bar/', run => 'my_app' };
+  # Will not reach this point as we executed my_app,
 
 =head1 DESCRIPTION
 
-This module lets you easily bundle a typical F<blib/> tree into a zip
-file, called a Perl Archive, or C<PAR>.
+This module lets you use special zip files, called B<P>erl B<Ar>chives, as
+libraries from which Perl modules can be loaded. 
 
 It supports loading XS modules by overriding B<DynaLoader> bootstrapping
 methods; it writes shared object file to a temporary file at the time it
 is needed.
 
-To generate a F<.par> file, all you have to do is compress the modules
-under F<arch/> and F<lib/>, e.g.:
+A F<.par> file is mostly a zip of the F<blib/> directory after the build
+process of a CPAN distribution. To generate a F<.par> file yourself, all
+you have to do is compress the modules under F<arch/> and F<lib/>, e.g.:
 
     % perl Makefile.PL
     % make
@@ -129,19 +139,21 @@ under F<arch/> and F<lib/>, e.g.:
     % zip -r mymodule.par arch/ lib/
 
 Afterward, you can just use F<mymodule.par> anywhere in your C<@INC>,
-use B<PAR>, and it will Just Work.
+use B<PAR>, and it will Just Work. Support for generating F<.par> files
+is going to be in the next (beyond 0.2805) release of Module::Build.
 
 For convenience, you can set the C<PERL5OPT> environment variable to
 C<-MPAR> to enable C<PAR> processing globally (the overhead is small
 if not used); setting it to C<-MPAR=/path/to/mylib.par> will load a
 specific PAR file.  Alternatively, consider using the F<par.pl> utility
-bundled with this module, or using the self-contained F<parl> utility
+bundled with the L<PAR::Packer> distribution, or using the
+self-contained F<parl> utility which is also distributed with L<PAR::Packer>
 on machines without PAR.pm installed.
 
 Note that self-containing scripts and executables created with F<par.pl>
 and F<pp> may also be used as F<.par> archives:
 
-    % pp -o packed.exe source.pl        # generate packed.exe
+    % pp -o packed.exe source.pl        # generate packed.exe (see PAR::Packer)
     % perl -MPAR=packed.exe other.pl    # this also works
     % perl -MPAR -Ipacked.exe other.pl  # ditto
 
@@ -162,7 +174,7 @@ attributes in F<META.yml>:
       verbatim: 0       # was packed prerequisite's PODs preserved?
       version: x.xx     # PAR.pm version that generated this PAR
 
-User-defined environment variables, like I<PAR_CLEAN>, always
+User-defined environment variables, like I<PAR_GLOBAL_CLEAN>, always
 overrides the ones set in F<META.yml>.  The algorithm for generating
 caching/temporary directory is as follows:
 
@@ -194,7 +206,8 @@ Here is a description of the variables the previous paths.
 
 I<TEMP> is a temporary directory, which can be set via 
 C<$ENV{PAR_GLOBAL_TMPDIR}>,
-C<$ENV{TMPDIR}>, C<$ENV{TEMP}> or C<$ENV{TMP}>, in that order of priority.
+C<$ENV{TMPDIR}>, C<$ENV{TEMPDIR}>, C<$ENV{TEMP}>
+or C<$ENV{TMP}>, in that order of priority.
 If none of those are set, I<C:\TEMP>, I</tmp> are checked.  If neither
 of them exists, I<.> is used.
 
@@ -297,6 +310,18 @@ sub import {
             "main.pl",
         );
 
+        if ($progname and !$member) {
+            require File::Spec;
+            my @path = File::Spec->splitdir($progname);
+            my $filename = pop @path;
+            $member = _first_member( $zip,
+                "script/".$filename,
+                "script/".$filename.".pl",
+                $filename,
+                $filename.".pl",
+            )
+        }
+
         # finally take $ARGV[0] as the hint for file to run
         if (defined $ARGV[0] and !$member) {
             $member = _first_member( $zip,
@@ -307,7 +332,9 @@ sub import {
             ) or die qq(PAR.pm: Can't open perl script "$ARGV[0]": No such file or directory);
             shift @ARGV;
         }
-        elsif (!$member) {
+
+
+        if (!$member) {
             die "Usage: $0 script_file_name.\n";
         }
 
@@ -355,12 +382,22 @@ sub _import_hash_ref {
     }
     else {
         # Deal with repositories elsewhere
-        return(_import_repository($opt));
+        my $client = _import_repository($opt);
+        return() if not $client;
+
+        if (defined $opt->{run}) {
+            # run was specified
+            # run the specified script from the repository
+            $client->run_script( $opt->{run} );
+            return 1;
+        }
+        
+        return 1;
     }
 
     # run was specified
     # run the specified script from inside the PAR file.
-    if (exists $opt->{run} and defined $opt->{run}) {
+    if (defined $opt->{run}) {
         my $script = $opt->{run};
         require PAR::Heavy;
         PAR::Heavy::_init_dynaloader();
@@ -388,17 +425,31 @@ sub _import_hash_ref {
 
 # This sub is invoked by _import_hash_ref if a {repository}
 # option is found
+# Returns the repository client object on success.
 sub _import_repository {
     my $opt = shift;
     my $url = $opt->{repository};
 
     eval "require PAR::Repository::Client; 1;";
-    if ($@) {
-        croak "In order to use the 'use PAR { repository => 'url' };' syntax, you need to install the PAR::Repository::Client module from CPAN. This module does not seem to be installed as indicated by the following error message: $@";
+    if ($@ or not PAR::Repository::Client->VERSION >= 0.04) {
+        croak "In order to use the 'use PAR { repository => 'url' };' syntax, you need to install the PAR::Repository::Client module (version 0.04 or later) from CPAN. This module does not seem to be installed as indicated by the following error message: $@";
     }
-    my $obj = PAR::Repository::Client->new(uri => $url);
+
+    my $obj;
+
+    # Support existing clients passed in as objects.
+    if (ref($url) and UNIVERSAL::isa($obj, 'PAR::Repository::Client')) {
+        $obj = $url;
+    }
+    else {
+        $obj = PAR::Repository::Client->new(
+            uri => $url,
+            auto_install => $opt->{install},
+        );
+    }
+
     push @RepositoryObjects, $obj;
-    return 1;
+    return $obj;
 }
 
 sub _first_member {
@@ -466,7 +517,8 @@ sub _extract_inc {
         rmdir("$inc.lock");
     }
 
-    unshift @INC, grep -d, map join('/', $inc, @$_),
+    require File::Spec;
+    unshift @INC, grep -d, map File::Spec->catdir($inc, @$_),
         [ 'lib' ], [ 'arch' ], [ $arch ], [ $ver ], [ $ver, $arch ], [];
 }
 
@@ -646,6 +698,49 @@ sub unpar {
             next unless $content =~ /^PK\003\004/;
             push @rv, unpar(\$content, undef, undef, 1);
         }
+        
+        # extract all dlls from the .par to $ENV{PAR_TEMP}!
+        # XXX is this correct?
+        # Intended to fix problem with Alien::wxWidgets/Wx...
+        # for all zip members that end in .dll/.so (depending on platform)
+        foreach my $member ( $zip->membersMatching(
+            '\.'.quotemeta($Config{dlext}).'$'
+        ) ) {
+            next if $member->isDirectory or !$ENV{PAR_TEMP};
+            my $member_name = $member->fileName;
+            next unless $member_name =~ m{
+                    \/([^/]+)$
+                }x
+                or $member_name =~ m{
+                    ^([^/]+)$
+                };
+            my $extract_name = $1;
+            my $dest_name = File::Spec->catfile($ENV{PAR_TEMP}, $extract_name);
+            $member->extractToFileNamed($dest_name);
+        }
+
+        # Now push this path into usual library search paths
+        my $separator = $Config{path_sep};
+        my $tempdir = $ENV{PAR_TEMP};
+        foreach my $key (qw(
+            LD_LIBRARY_PATH
+            LIB_PATH
+            LIBRARY_PATH
+            PATH
+            DYLD_LIBRARY_PATH
+        )) {
+           if (defined $ENV{$key} and $ENV{$key} ne '') {
+               # Check whether it's already in the path. If so, don't
+               # append the PAR temp dir in order not to overflow the
+               # maximum length for ENV vars.
+               $ENV{$key} .= $separator . $tempdir
+                 unless grep { $_ eq $tempdir } split $separator, $ENV{$key};
+           }
+           else {
+               $ENV{$key} = $tempdir;
+           }
+       }
+    
     }
 
     $LastAccessedPAR = $zip;
@@ -685,13 +780,24 @@ sub _set_par_temp {
     require File::Spec;
 
     foreach my $path (
-        (map $ENV{$_}, qw( PAR_TMPDIR TMPDIR TEMP TMP )),
+        (map $ENV{$_}, qw( PAR_TMPDIR TMPDIR TEMPDIR TEMP TMP )),
         qw( C:\\TEMP /tmp . )
     ) {
         next unless $path and -d $path and -w $path;
-        my $username = defined(&Win32::LoginName)
-            ? &Win32::LoginName()
-            : $ENV{USERNAME} || $ENV{USER} || 'SYSTEM';
+        my $username;
+        my $pwuid;
+        # does not work everywhere:
+        eval {($pwuid) = getpwuid($>) if defined $>;};
+
+        if ( defined(&Win32::LoginName) ) {
+            $username = &Win32::LoginName;
+        }
+        elsif (defined $pwuid) {
+            $username = $pwuid;
+        }
+        else {
+            $username = $ENV{USERNAME} || $ENV{USER} || 'SYSTEM';
+        }
         $username =~ s/\W/_/g;
 
         my $stmpdir = File::Spec->catdir($path, "par-$username");
@@ -805,7 +911,8 @@ The PAR homepage at L<http://par.perl.org>.
 L<PAR::Tutorial>, L<PAR::FAQ> (For a more
 current FAQ, refer to the homepage.)
 
-L<par.pl>, L<parl>, L<pp>
+The L<PAR::Packer> distribution which contains the packaging utilities:
+L<par.pl>, L<parl>, L<pp>.
 
 L<PAR::Dist> for details on PAR distributions.
 
